use crate::Point;
use std::ops::{Add, AddAssign};
use num::Integer;

/*
P + T
P + (T, T)
P + P
P + &P
P + &mP
&P + P
&mP + P
&P + T
&P + (T, T)
&mP + T
&mP + (T, T)
&P + &P
&P + &mP
&mP + &P
&mP + &mP
P += T
P += (T, T)
P += P
P += &P
P += &mP
*/

impl<T: Integer + Copy> Add<T> for Point<T> {
    type Output = Self;
    fn add(self, rhs: T) -> Self::Output {
        Self::from((self.x + rhs, self.y + rhs))
    }
}

impl<T: Integer> Add<(T, T)> for Point<T> {
    type Output = Self;
    fn add(self, rhs: (T, T)) -> Self::Output {
        Self::from((self.x+rhs.0, self.y+rhs.1))
    }
}

impl<T: Integer> Add<Point<T>> for Point<T> {
    type Output = Self;
    fn add(self, rhs: Point<T>) -> Self::Output {
        Self::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<&Point<T>> for Point<T> {
    type Output = Self;
    fn add(self, rhs: &Point<T>) -> Self::Output {
        Self::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<&mut Point<T>> for Point<T> {
    type Output = Self;
    fn add(self, rhs: &mut Point<T>) -> Self::Output {
        Self::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: Point<T>) -> Self::Output {
        Point::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: Point<T>) -> Self::Output {
        Point::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<T> for &Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: T) -> Self::Output {
        Point::from((self.x+rhs, self.y+rhs))
    }
}

impl<T: Integer + Copy> Add<(T, T)> for &Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: (T, T)) -> Self::Output {
        Point::from((self.x+rhs.0, self.y+rhs.1))
    }
}

impl<T: Integer + Copy> Add<&Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: &Point<T>) -> Self::Output {
        Point::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<&mut Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: &mut Point<T>) -> Self::Output {
        Point::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<&Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: &Point<T>) -> Self::Output {
        Point::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<&mut Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: &mut Point<T>) -> Self::Output {
        Point::from((self.x+rhs.x, self.y+rhs.y))
    }
}

impl<T: Integer + Copy> Add<T> for &mut Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: T) -> Self::Output {
        Point::from((self.x+rhs, self.y+rhs))
    }
}

impl<T: Integer + Copy> Add<(T, T)> for &mut Point<T> {
    type Output = Point<T>;
    fn add(self, rhs: (T, T)) -> Self::Output {
        Point::from((self.x+rhs.0, self.y+rhs.1))
    }
}


impl<T: Integer + AddAssign + Copy> AddAssign<T> for Point<T> {
    fn add_assign(&mut self, rhs: T) {
        self.x += rhs;
        self.y += rhs;
    }
}

impl<T: Integer + AddAssign> AddAssign<(T, T)> for Point<T> {
    fn add_assign(&mut self, rhs: (T, T)) {
        self.x += rhs.0;
        self.y += rhs.1;
    }
}

impl<T: Integer + AddAssign + Copy> AddAssign<Point<T>> for Point<T> {
    fn add_assign(&mut self, rhs: Point<T>) {
        self.x += rhs.x;
        self.y += rhs.y;
    }
}

impl<T: Integer + AddAssign + Copy> AddAssign<&Point<T>> for Point<T> {
    fn add_assign(&mut self, rhs: &Point<T>) {
        self.x += rhs.x;
        self.y += rhs.y;
    }
}

impl<T: Integer + AddAssign + Copy> AddAssign<&mut Point<T>> for Point<T> {
    fn add_assign(&mut self, rhs: &mut Point<T>) {
        self.x += rhs.x;
        self.y += rhs.y;
    }
}
