use crate::Point;
use std::ops::{Div, DivAssign};
use num::Integer;

/*
P / T
P / (T, T)
P / P
P / &P
P / &mP
&P / T
&P / (T, T)
&P / P
&P / &P
&P / &mP
&mP / T
&mP / (T, T)
&mP / P
&mP / &P
&mP / &mP
P /= T
P /= (T, T)
P /= P
P /= &P
P /= &mP
*/

impl<T: Integer + Copy> Div<T> for Point<T> {
    type Output = Self;
    fn div(self, rhs: T) -> Self::Output {
        Self::from((self.x/rhs, self.y/rhs))
    }
}

impl<T: Integer> Div<(T, T)> for Point<T> {
    type Output = Self;
    fn div(self, rhs: (T, T)) -> Self::Output {
        Self::from((self.x/rhs.0, self.y/rhs.1))
    }
}

impl<T: Integer> Div<Point<T>> for Point<T> {
    type Output = Self;
    fn div(self, rhs: Point<T>) -> Self::Output {
        Self::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<&Point<T>> for Point<T> {
    type Output = Self;
    fn div(self, rhs: &Point<T>) -> Self::Output {
        Self::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<&mut Point<T>> for Point<T> {
    type Output = Self;
    fn div(self, rhs: &mut Point<T>) -> Self::Output {
        Self::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<T> for &Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: T) -> Self::Output {
        Point::from((self.x/rhs, self.y/rhs))
    }
}

impl<T: Integer + Copy> Div<(T, T)> for &Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: (T, T)) -> Self::Output {
        Point::from((self.x/rhs.0, self.y/rhs.1))
    }
}

impl<T: Integer + Copy> Div<Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: Point<T>) -> Self::Output {
        Point::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<&Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: &Point<T>) -> Self::Output {
        Point::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<&mut Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: &mut Point<T>) -> Self::Output {
        Point::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<T> for &mut Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: T) -> Self::Output {
        Point::from((self.x/rhs, self.y/rhs))
    }
}

impl<T: Integer + Copy> Div<(T, T)> for &mut Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: (T, T)) -> Self::Output {
        Point::from((self.x/rhs.0, self.y/rhs.1))
    }
}

impl<T: Integer + Copy> Div<Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: Point<T>) -> Self::Output {
        Point::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<&Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: &Point<T>) -> Self::Output {
        Point::from((self.x/rhs.x, self.y/rhs.y))
    }
}

impl<T: Integer + Copy> Div<&mut Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn div(self, rhs: &mut Point<T>) -> Self::Output {
        Point::from((self.x/rhs.x, self.y/rhs.y))
    }
}


impl<T: Integer + DivAssign + Copy> DivAssign<T> for Point<T> {
    fn div_assign(&mut self, rhs: T) {
        self.x /= rhs;
        self.y /= rhs;
    }
}

impl<T: Integer + DivAssign> DivAssign<(T, T)> for Point<T> {
    fn div_assign(&mut self, rhs: (T, T)) {
        self.x /= rhs.0;
        self.y /= rhs.1;
    }
}

impl<T: Integer + DivAssign + Copy> DivAssign<Point<T>> for Point<T> {
    fn div_assign(&mut self, rhs: Point<T>) {
        self.x /= rhs.x;
        self.y /= rhs.y;
    }
}

impl<T: Integer + DivAssign + Copy> DivAssign<&Point<T>> for Point<T> {
    fn div_assign(&mut self, rhs: &Point<T>) {
        self.x /= rhs.x;
        self.y /= rhs.y;
    }
}

impl<T: Integer + DivAssign + Copy> DivAssign<&mut Point<T>> for Point<T> {
    fn div_assign(&mut self, rhs: &mut Point<T>) {
        self.x /= rhs.x;
        self.y /= rhs.y;
    }
}
