use crate::Point;
use std::ops::{Mul, MulAssign};
use num::Integer;

/*
P * T
P * (T, T)
P * P
P * &P
P * &mP
&P * T
&P * (T, T)
&P * P
&P * &P
&P * &mP
&mP * T
&mP * (T, T)
&mP * P
&mP * &P
&mP * &mP
P *= T
P *= (T, T)
P *= P
P *= &P
P *= &mP
*/

impl<T: Integer + Copy> Mul<T> for Point<T> {
    type Output = Self;
    fn mul(self, rhs: T) -> Self::Output {
        Self::from((self.x*rhs, self.y*rhs))
    }
}

impl<T: Integer> Mul<(T, T)> for Point<T> {
    type Output = Self;
    fn mul(self, rhs: (T, T)) -> Self::Output {
        Self::from((self.x*rhs.0, self.y*rhs.1))
    }
}

impl<T: Integer> Mul<Point<T>> for Point<T> {
    type Output = Self;
    fn mul(self, rhs: Point<T>) -> Self::Output {
        Self::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<&Point<T>> for Point<T> {
    type Output = Self;
    fn mul(self, rhs: &Point<T>) -> Self::Output {
        Self::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<&mut Point<T>> for Point<T> {
    type Output = Self;
    fn mul(self, rhs: &mut Point<T>) -> Self::Output {
        Self::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<T> for &Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: T) -> Self::Output {
        Point::from((self.x*rhs, self.y*rhs))
    }
}

impl<T: Integer + Copy> Mul<(T, T)> for &Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: (T, T)) -> Self::Output {
        Point::from((self.x*rhs.0, self.y*rhs.1))
    }
}

impl<T: Integer + Copy> Mul<Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: Point<T>) -> Self::Output {
        Point::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<&Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: &Point<T>) -> Self::Output {
        Point::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<&mut Point<T>> for &Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: &mut Point<T>) -> Self::Output {
        Point::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<T> for &mut Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: T) -> Self::Output {
        Point::from((self.x*rhs, self.y*rhs))
    }
}

impl<T: Integer + Copy> Mul<(T, T)> for &mut Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: (T, T)) -> Self::Output {
        Point::from((self.x*rhs.0, self.y*rhs.1))
    }
}

impl<T: Integer + Copy> Mul<Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: Point<T>) -> Self::Output {
        Point::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<&Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: &Point<T>) -> Self::Output {
        Point::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + Copy> Mul<&mut Point<T>> for &mut Point<T> {
    type Output = Point<T>;
    fn mul(self, rhs: &mut Point<T>) -> Self::Output {
        Point::from((self.x*rhs.x, self.y*rhs.y))
    }
}

impl<T: Integer + MulAssign + Copy> MulAssign<T> for Point<T> {
    fn mul_assign(&mut self, rhs: T) {
        self.x *= rhs;
        self.y *= rhs;
    }
}

impl<T: Integer + MulAssign> MulAssign<(T, T)> for Point<T> {
    fn mul_assign(&mut self, rhs: (T, T)) {
        self.x *= rhs.0;
        self.y *= rhs.1;
    }
}

impl<T: Integer + MulAssign + Copy> MulAssign<Point<T>> for Point<T> {
    fn mul_assign(&mut self, rhs: Point<T>) {
        self.x *= rhs.x;
        self.y *= rhs.y;
    }
}

impl<T: Integer + MulAssign + Copy> MulAssign<&Point<T>> for Point<T> {
    fn mul_assign(&mut self, rhs: &Point<T>) {
        self.x *= rhs.x;
        self.y *= rhs.y;
    }
}

impl<T: Integer + MulAssign + Copy> MulAssign<&mut Point<T>> for Point<T> {
    fn mul_assign(&mut self, rhs: &mut Point<T>) {
        self.x *= rhs.x;
        self.y *= rhs.y;
    }
}
