use crate::Point;

#[test]
fn default() {
    let p = Point::<i32>::default();
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 0, "p.y={:?} expected=0", p.y);

    let p = Point::<u32>::default();
    assert_eq!(p.x, 0, "p.x={:?} expected=0.0", p.x);
    assert_eq!(p.y, 0, "p.y={:?} expected=0.0", p.y);
}

#[test]
fn from_single() {
    let p = Point::from(3i8);
    assert_eq!(p.x, 3, "p.x={:?} expected=3", p.x);
    assert_eq!(p.y, 3, "p.y={:?} expected=3", p.y);

    let p = Point::from(4u64);
    assert_eq!(p.x, 4, "p.x={:?} expected=4", p.x);
    assert_eq!(p.y, 4, "p.y={:?} expected=4", p.y);
}

#[test]
fn from_tuple() {
    let p = Point::from((1i32, 2));
    assert_eq!(p.x, 1, "p.x={:?} expected=1", p.x);
    assert_eq!(p.y, 2, "p.y={:?} expected=2", p.y);

    let p = Point::from((3u8, 8));
    assert_eq!(p.x, 3, "p.x={:?} expected=3", p.x);
    assert_eq!(p.y, 8, "p.y={:?} expected=8", p.y);
}

#[test]
fn try_from_string() -> Result<(), anyhow::Error> {
    let p = Point::<i32>::try_from("(1,2)".to_string())?;
    assert_eq!(p.x, 1, "p.x={:?} expected=1", p.x);
    assert_eq!(p.y, 2, "p.y={:?} expected=2", p.y);
    Ok(())
}

#[test]
fn to_string() {
    let p = Point::from((1, 2));
    assert_eq!(p.to_string(), "(1, 2)");
}

#[test]
fn display() {
    let p = Point::from((3i8, 6));
    assert_eq!(format!("{}", p), "(3, 6)");
}

#[test]
fn add_single() {
    let p = Point::from((1, 2));

    let p2 = &p + 2;
    assert_eq!(p2.x, 3, "p.x={:?} expected=3", p2.x);
    assert_eq!(p2.y, 4, "p.y={:?} expected=4", p2.y);

    let p2 = p + 4;
    assert_eq!(p2.x, 5, "p.x={:?} expected=5", p2.x);
    assert_eq!(p2.y, 6, "p.y={:?} expected=6", p2.y);
}

#[test]
fn add_tuple() {
    let p = Point::from((1, 2));

    let p2 = &p + (1, 2);
    assert_eq!(p2.x, 2, "p.x={:?} expected=2", p2.x);
    assert_eq!(p2.y, 4, "p.y={:?} expected=4", p2.y);
    let p2 = p + (-1, -2);
    assert_eq!(p2.x, 0, "p.x={:?} expected=0", p2.x);
    assert_eq!(p2.y, 0, "p.y={:?} expected=0", p2.y);
}

#[test]
fn sub_single() {
    let p = Point::from((1,2));

    let p2 = &p - 1;
    assert_eq!(p2.x, 0, "p.x={:?} expected=0", p2.x);
    assert_eq!(p2.y, 1, "p.y={:?} expected=1", p2.y);

    let p2 = p - 2;
    assert_eq!(p2.x, -1, "p.x={:?} expected=-1", p2.x);
    assert_eq!(p2.y, 0, "p.y={:?} expected=0", p2.y);
}

#[test]
fn sub_tuple() {
    let p = Point::from((2, 3));

    let p2 = &p - (1, 2);
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 1, "p.y={:?} expected=1", p2.y);

    let p2 = p - (-1, -2);
    assert_eq!(p2.x, 3, "p.x={:?} expected=3", p2.x);
    assert_eq!(p2.y, 5, "p.y={:?} expected=5", p2.y);
}

#[test]
fn mul_single() {
    let p = Point::from((1, 2));

    let p2 = &p * 2;
    assert_eq!(p2.x, 2, "p.x={:?} expected=2", p2.x);
    assert_eq!(p2.y, 4, "p.y={:?} expected=4", p2.y);

    let p2 = p * -2;
    assert_eq!(p2.x, -2, "p.x={:?} expected=-2", p2.x);
    assert_eq!(p2.y, -4, "p.y={:?} expected=-4", p2.y);
}

#[test]
fn mul_tuple() {
    let p = Point::from((1, 2));

    let p2 = &p * (1, 3);
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 6, "p.y={:?} expected=6", p2.y);

    let p2 = p * (-1, 4);
    assert_eq!(p2.x, -1, "p.x={:?} expected=-1", p2.x);
    assert_eq!(p2.y, 8, "p.y={:?} expected=8", p2.y);
}

#[test]
fn div_single() {
    let p = Point::from((1, 2));

    let p2 = &p / 2;
    assert_eq!(p2.x, 0, "p.x={:?} expected=0", p2.x);
    assert_eq!(p2.y, 1, "p.y={:?} expected=1", p2.y);

    let p2 = p / 1;
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 2, "p.y={:?} expected=2", p2.y);
}

#[test]
fn div_tuple() {
    let p = Point::from((2, 4));

    let p2 = &p / (2, 2);
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 2, "p.y={:?} expected=2", p2.y);

    let p2 = p / (2, 4);
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 1, "p.y={:?} expected=1", p2.y);
}

#[test]
fn rem_single() {
    let p = Point::from((1, 2));

    let p2 = &p % 2;
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 0, "p.y={:?} expected=0", p2.y);

    let p2 = p % 3;
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 2, "p.y={:?} expected=2", p2.y);
}

#[test]
fn rem_tuple() {
    let p = Point::from((1, 2));

    let p2 = &p % (2, 2);
    assert_eq!(p2.x, 1, "p.x={:?} expected=1", p2.x);
    assert_eq!(p2.y, 0, "p.y={:?} expected=0", p2.y);

    let p2 = p % (1, 1);
    assert_eq!(p2.x, 0, "p.x={:?} expected=0", p2.x);
    assert_eq!(p2.y, 0, "p.y={:?} expected=0", p2.y);
}

/// Method to reset a Point because of Assign
///
/// Used for std::ops
/// - AddAssign (+=)
/// - SubAssign (-=)
/// - MulAssign (*=)
/// - DivAssign (/=)
/// - RemAssign (%=)
fn reset(p: &mut Point<i32>) {
    p.x = 1;
    p.y = 2;
}

#[test]
fn add_assign_single() {
    let mut p = Point::from((1, 2));

    p += 1;
    assert_eq!(p.x, 2, "p.x={:?} expected=2", p.x);
    assert_eq!(p.y, 3, "p.y={:?} expected=3", p.y);
    reset(&mut p);

    p += -1;
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 1, "p.y={:?} expected=1", p.y);
}

#[test]
fn add_assign_tuple() {
    let mut p = Point::from((1, 2));

    p += (1, 2);
    assert_eq!(p.x, 2, "p.x={:?} expected=2", p.x);
    assert_eq!(p.y, 4, "p.y={:?} expected=4", p.y);
    reset(&mut p);

    p += (2, 4);
    assert_eq!(p.x, 3, "p.x={:?} expected=3", p.x);
    assert_eq!(p.y, 6, "p.y={:?} expected=6", p.y);
}

#[test]
fn sub_assign_single() {
    let mut p = Point::from((1, 2));

    p -= 1;
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 1, "p.y={:?} expected=1", p.y);
    reset(&mut p);

    p -= -1;
    assert_eq!(p.x, 2, "p.x={:?} expected=2", p.x);
    assert_eq!(p.y, 3, "p.y={:?} expected=3", p.y);
}

#[test]
fn sub_assign_tuple() {
    let mut p = Point::from((1, 2));

    p -= (1, 2);
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 0, "p.y={:?} expected=0", p.y);
    reset(&mut p);

    p -= (-1, -2);
    assert_eq!(p.x, 2, "p.x={:?} expected=2", p.x);
    assert_eq!(p.y, 4, "p.y={:?} expected=4", p.y);
}

#[test]
fn mul_assign_single() {
    let mut p = Point::from((1, 2));

    p *= 2;
    assert_eq!(p.x, 2, "p.x={:?} expected=2", p.x);
    assert_eq!(p.y, 4, "p.y={:?} expected=4", p.y);
    reset(&mut p);

    p *= 3;
    assert_eq!(p.x, 3, "p.x={:?} expected=3", p.x);
    assert_eq!(p.y, 6, "p.y={:?} expected=6", p.y);
}

#[test]
fn mul_assign_tuple() {
    let mut p = Point::from((1, 2));

    p *= (2, 3);
    assert_eq!(p.x, 2, "p.x={:?} expected=2", p.x);
    assert_eq!(p.y, 6, "p.y={:?} expected=6", p.y);
    reset(&mut p);

    p *= (3, -3);
    assert_eq!(p.x, 3, "p.x={:?} expected=3", p.x);
    assert_eq!(p.y, -6, "p.y={:?} expected=-6", p.y);
}

#[test]
fn div_assign_single() {
    let mut p = Point::from((1, 2));

    p /= 2;
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 1, "p.y={:?} expected=1", p.y);
    reset(&mut p);

    p /= 3;
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 0, "p.y={:?} expected=0", p.y);
}

#[test]
fn div_assign_tuple() {
    let mut p = Point::from((1, 2));

    p /= (2, 1);
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 2, "p.y={:?} expected=2", p.y);
    reset(&mut p);

    p /= (1, 2);
    assert_eq!(p.x, 1, "p.x={:?} expected=1", p.x);
    assert_eq!(p.y, 1, "p.y={:?} expected=1", p.y);
}

#[test]
fn rem_assign_single() {
    let mut p = Point::from((1, 2));

    p %= 2;
    assert_eq!(p.x, 1, "p.x={:?} expected=1", p.x);
    assert_eq!(p.y, 0, "p.y={:?} expected=0", p.y);
    reset(&mut p);

    p %= 3;
    assert_eq!(p.x, 1, "p.x={:?} expected=1", p.x);
    assert_eq!(p.y, 2, "p.y={:?} expected=2", p.y);
}

#[test]
fn rem_assign_tuple() {
    let mut p = Point::from((1, 2));

    p %= (1, 2);
    assert_eq!(p.x, 0, "p.x={:?} expected=0", p.x);
    assert_eq!(p.y, 0, "p.y={:?} expected=0", p.y);
    reset(&mut p);

    p %= (-3, -3);
    assert_eq!(p.x, 1, "p.x={:?} expected=1", p.x);
    assert_eq!(p.y, 2, "p.y={:?} expected=2", p.y);
}

#[test]
fn neg() {
    let p = Point::from((1, -2));

    let p1 = -p;
    assert_eq!(p1.x, -1, "p.x={:?} expected=-1", p1.x);
    assert_eq!(p1.y, 2, "p.x={:?} expected=2", p1.y);
}
